import 'dart:convert';

import 'package:equatable/equatable.dart';

// ignore_for_file: non_constant_identifier_names

/// Token model
class Token extends Equatable {
  final int version;
  final int expires_in;
  final String username;
  final String token_type;
  final String access_token;
  final String refresh_token;
  final String? type;
  final String? email;
  final String? fullName;
  final String? tenantName;
  final String? locale;

  const Token({
    required this.version,
    required this.expires_in,
    required this.username,
    required this.token_type,
    required this.access_token,
    required this.refresh_token,
    this.type,
    this.email,
    this.fullName,
    this.tenantName,
    this.locale,
  });

  /// Empty token which represents an unauthenticated user.
  static const empty = Token(
    version: 0,
    expires_in: 0,
    username: '',
    token_type: '',
    access_token: '',
    refresh_token: '',
    type: '',
    email: '',
    fullName: '',
    tenantName: '',
    locale: '',
  );

  /// Convenience getter to determine whether the current token is empty.
  bool get isEmpty => this == Token.empty;

  /// Convenience getter to determine whether the current token is not empty.
  bool get isNotEmpty => this != Token.empty;

  Token copyWith({
    int? version,
    int? expires_in,
    String? username,
    String? token_type,
    String? access_token,
    String? refresh_token,
    String? type,
    String? email,
    String? fullName,
    String? tenantName,
    String? locale,
  }) {
    return Token(
      version: version ?? this.version,
      expires_in: expires_in ?? this.expires_in,
      username: username ?? this.username,
      token_type: token_type ?? this.token_type,
      access_token: access_token ?? this.access_token,
      refresh_token: refresh_token ?? this.refresh_token,
      type: type ?? this.type,
      email: email ?? this.email,
      fullName: fullName ?? this.fullName,
      tenantName: tenantName ?? this.tenantName,
      locale: locale ?? this.locale,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'version': version,
      'expires_in': expires_in,
      'username': username,
      'token_type': token_type,
      'access_token': access_token,
      'refresh_token': refresh_token,
      'type': type,
      'email': email,
      'fullName': fullName,
      'tenantName': tenantName,
      'locale': locale,
    };
  }

  factory Token.fromMap(Map<String, dynamic> map) {
    return Token(
      version: map['version'],
      expires_in: map['expires_in'],
      username: map['username'],
      token_type: map['token_type'],
      access_token: map['access_token'],
      refresh_token: map['refresh_token'],
      type: map['type'],
      email: map['email'],
      fullName: map['fullName'],
      tenantName: map['tenantName'],
      locale: map['locale'],
    );
  }

  String toJson() => json.encode(toMap());

  factory Token.fromJson(String source) => Token.fromMap(json.decode(source));

  @override
  bool get stringify => true;

  @override
  List<Object?> get props {
    return [
      version,
      expires_in,
      username,
      token_type,
      access_token,
      refresh_token,
      type,
      email,
      fullName,
      tenantName,
      locale,
    ];
  }
}
