import 'dart:convert';

import 'package:equatable/equatable.dart';

import 'user_integration.dart';

class User extends Equatable {
  final bool changePassword;
  final String? photo;
  final bool admin;
  final bool allowedToChangePassword;
  final String id;
  final String username;
  final String fullName;
  final String email;
  final String? locale;
  final String tenantDomain;
  final String tenantName;
  final String tenantLocale;
  final bool blocked;
  final String? authenticationType;
  final String? description;
  final List<dynamic>? properties;
  final Integration? integration;
  final String? discriminator;

  User({
    required this.changePassword,
    this.photo,
    required this.admin,
    required this.allowedToChangePassword,
    required this.id,
    required this.username,
    required this.fullName,
    required this.email,
    this.locale,
    required this.tenantDomain,
    required this.tenantName,
    required this.tenantLocale,
    required this.blocked,
    this.authenticationType,
    this.description,
    this.properties,
    this.integration,
    this.discriminator,
  });

  User copyWith({
    bool? changePassword,
    String? photo,
    bool? admin,
    bool? allowedToChangePassword,
    String? id,
    String? username,
    String? fullName,
    String? email,
    String? locale,
    String? tenantDomain,
    String? tenantName,
    String? tenantLocale,
    bool? blocked,
    String? authenticationType,
    String? description,
    List<dynamic>? properties,
    Integration? integration,
    String? discriminator,
  }) {
    return User(
      changePassword: changePassword ?? this.changePassword,
      photo: photo ?? this.photo,
      admin: admin ?? this.admin,
      allowedToChangePassword:
          allowedToChangePassword ?? this.allowedToChangePassword,
      id: id ?? this.id,
      username: username ?? this.username,
      fullName: fullName ?? this.fullName,
      email: email ?? this.email,
      locale: locale ?? this.locale,
      tenantDomain: tenantDomain ?? this.tenantDomain,
      tenantName: tenantName ?? this.tenantName,
      tenantLocale: tenantLocale ?? this.tenantLocale,
      blocked: blocked ?? this.blocked,
      authenticationType: authenticationType ?? this.authenticationType,
      description: description ?? this.description,
      properties: properties ?? this.properties,
      integration: integration ?? this.integration,
      discriminator: discriminator ?? this.discriminator,
    );
  }

  Map<String, dynamic> toMap() {
    return {
      'changePassword': changePassword,
      'photo': photo,
      'admin': admin,
      'allowedToChangePassword': allowedToChangePassword,
      'id': id,
      'username': username,
      'fullName': fullName,
      'email': email,
      'locale': locale,
      'tenantDomain': tenantDomain,
      'tenantName': tenantName,
      'tenantLocale': tenantLocale,
      'blocked': blocked,
      'authenticationType': authenticationType,
      'description': description,
      'properties': properties,
      'integration': integration?.toMap(),
      'discriminator': discriminator,
    };
  }

  factory User.fromMap(Map<String, dynamic> map) {
    return User(
      changePassword: map['changePassword'],
      photo: map['photo'],
      admin: map['admin'],
      allowedToChangePassword: map['allowedToChangePassword'],
      id: map['id'],
      username: map['username'],
      fullName: map['fullName'],
      email: map['email'],
      locale: map['locale'],
      tenantDomain: map['tenantDomain'],
      tenantName: map['tenantName'],
      tenantLocale: map['tenantLocale'],
      blocked: map['blocked'],
      authenticationType: map['authenticationType'],
      description: map['description'],
      properties: List<dynamic>.from(map['properties']),
      integration: Integration.fromMap(map['integration']),
      discriminator: map['discriminator'],
    );
  }

  String toJson() => json.encode(toMap());

  factory User.fromJson(String source) => User.fromMap(json.decode(source));

  @override
  bool get stringify => true;

  @override
  List<Object?> get props {
    return [
      changePassword,
      photo,
      admin,
      allowedToChangePassword,
      id,
      username,
      fullName,
      email,
      locale,
      tenantDomain,
      tenantName,
      tenantLocale,
      blocked,
      authenticationType,
      description,
      properties,
      integration,
      discriminator,
    ];
  }
}
